//
//  AlfordCTSCTPeripheral.m
//  nRF UART
//
//  Created by Ole Morten on 1/12/13.
//  Copyright (c) 2013 Nordic Semiconductor. All rights reserved.
//

#import "AlfordCTSCTPeripheral.h"


@interface AlfordCTSCTPeripheral ()
@property CBService *CTSService;
@property CBCharacteristic *AlarmClkCharacteristic;
@property CBCharacteristic *CurrentTimeCharacteristic;

@end

@implementation AlfordCTSCTPeripheral
@synthesize peripheral = _peripheral;
@synthesize delegate = _delegate;
@synthesize CM;
@synthesize peripherals;
@synthesize CTSService = _CTSService;
@synthesize AlarmClkCharacteristic = _AlarmClkCharacteristic;
@synthesize CurrentTimeCharacteristic = _CurrentTimeCharacteristic;

+ (CBUUID *) CTSServiceUUID
{
    return [CBUUID UUIDWithString:@"6e400001-b5a3-f393-e0a9-e50e24dcca9e"];
}

+ (CBUUID *) CurrentTimeCharacteristicUUID
{
    return [CBUUID UUIDWithString:@"2A2B"];
    //return [CBUUID UUIDWithString:@"6e400002-b5a3-f393-e0a9-e50e24dcca9e"];
}

+ (CBUUID *) AlarmCharacteristicUUID
{
    return [CBUUID UUIDWithString:@"6e400002-b5a3-f393-e0a9-e50e24dcca9e"];
}

+ (CBUUID *) deviceInformationServiceUUID
{
    return [CBUUID UUIDWithString:@"180A"];
}

+ (CBUUID *) hardwareRevisionStringUUID
{
    return [CBUUID UUIDWithString:@"2A27"];
}


- (AlfordCTSCTPeripheral *) initWithPeripheral:(CBPeripheral*)peripheral delegate:(id<AlfordCTSCTPeripheralDelegate>) delegate
{
    if (self = [super init])
    {
        _peripheral = peripheral;
        _peripheral.delegate = self;
        _delegate = delegate;
    }
    return self;
}

- (void) didConnect
{
    
    [_peripheral discoverServices:@[self.class.CTSServiceUUID, self.class.deviceInformationServiceUUID]];
    NSLog(@"Did start service discovery.");
}

- (void) didDisconnect
{

}

- (void) writeAlarm:(NSData *) data
{
    [self.peripheral writeValue:data forCharacteristic:self.AlarmClkCharacteristic type:CBCharacteristicWriteWithResponse ];
}

- (void) writeCurrentTime:(NSData *) data
{
    NSLog(@"writeCurrentTime %@", data);
    [self.peripheral writeValue:data forCharacteristic:self.CurrentTimeCharacteristic type:CBCharacteristicWriteWithResponse ];
}

- (void) peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    if (error)
    {
        NSLog(@"Error discovering services: %@", error);
        return;
    }
    
    for (CBService *s in [peripheral services])
    {
        if ([s.UUID isEqual:self.class.CTSServiceUUID])
        {
            NSLog(@"Found correct service");
            self.CTSService = s;
            
            [self.peripheral discoverCharacteristics:@[self.class.CurrentTimeCharacteristicUUID, self.class.AlarmCharacteristicUUID] forService:self.CTSService];
        }
        else if ([s.UUID isEqual:self.class.deviceInformationServiceUUID])
        {
            [self.peripheral discoverCharacteristics:@[self.class.hardwareRevisionStringUUID] forService:s];
        }
    }
}

- (void) peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
{
    if (error)
    {
        NSLog(@"Error discovering characteristics: %@", error);
        return;
    }
    
    
    for (CBCharacteristic *c in [service characteristics])
    {
        NSLog(@"discovering characteristics: %@", c.UUID);
        if ([c.UUID isEqual:self.class.AlarmCharacteristicUUID])
        {
            NSLog(@"Found Alarm Clock characteristic");
            self.AlarmClkCharacteristic = c;
            
            //[self.peripheral setNotifyValue:YES forCharacteristic:self.AlarmClkCharacteristic];
        }
        else if ([c.UUID isEqual:self.class.CurrentTimeCharacteristicUUID])
        {
            NSLog(@"Found Current Time characteristic");
            self.CurrentTimeCharacteristic = c;
            [self.peripheral readValueForCharacteristic:c];

        }
        else if ([c.UUID isEqual:self.class.hardwareRevisionStringUUID])
        {
            NSLog(@"Found Hardware Revision String characteristic");
        }
    }
}

- (void) peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    if (error)
    {
        NSLog(@"Error receiving notification for characteristic %@: %@", characteristic, error);
        return;
    }
    
    NSLog(@"Received data on a characteristic.");
    
    if (characteristic == self.AlarmClkCharacteristic)
    {
        
        NSString *string = [NSString stringWithUTF8String:[[characteristic value] bytes]];
        [self.delegate didReceiveData:string];
    }
    else if ([characteristic.UUID isEqual:self.class.hardwareRevisionStringUUID])
    {
        NSString *hwRevision = @"";
        const uint8_t *bytes = characteristic.value.bytes;
        for (int i = 0; i < characteristic.value.length; i++)
        {
            NSLog(@"%x", bytes[i]);
            hwRevision = [hwRevision stringByAppendingFormat:@"0x%02x, ", bytes[i]];
        }
        
        [self.delegate didReadHardwareRevisionString:[hwRevision substringToIndex:hwRevision.length-2]];
    }
}

@end
