//
//  ViewController.m
//  nRF UART
//
//  Created by Ole Morten on 1/11/13.
//  Copyright (c) 2013 Nordic Semiconductor. All rights reserved.
//

#import "ViewController.h"
#define BLE_WATCH_NAME    "Moto Watch"
NSString *mixarray;
NSString *sendtext1;
NSString *sendtext2;
int sendint1;
int sendint2;
typedef enum
{
    IDLE = 0,
    SCANNING,
    CONNECTED,
} ConnectionState;

typedef enum
{
    LOGGING,
    RX,
    TX,
} ConsoleDataType;

NSTimer *reconntimer;

@interface ViewController ()
@property CBCentralManager *cm;
@property ConnectionState state;
@property AlfordCTSCTPeripheral *currentPeripheral;
@end

@implementation ViewController
@synthesize cm = _cm;
@synthesize currentPeripheral = _currentPeripheral;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.cm = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
    
    [self addTextToConsole:@"Did start application" dataType:LOGGING];
    
    [self.tableView setSeparatorStyle:UITableViewCellSeparatorStyleNone];
    [self.tableView setSeparatorColor:[UIColor clearColor]];
    
    [self.sendTextField setDelegate:self];
    [self.sendTextField2 setDelegate:self];
    
    [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(check) userInfo:(nil) repeats:(YES)];
    [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(check2) userInfo:(nil) repeats:(YES)];
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDirPath = [paths objectAtIndex:0];
    pairInfopath = [docDirPath stringByAppendingPathComponent:@"BLE_Pair_watch.plist"];
    NSFileManager *pairInfoFM = [NSFileManager defaultManager];
    
    if ([pairInfoFM fileExistsAtPath:pairInfopath])
    {
        NSLog(@"BLE_Pair.plist create by path exist!" );
        pairedDeviceDict = [[NSMutableDictionary alloc] initWithContentsOfFile:pairInfopath];
        self.DelBond.enabled =TRUE;
    }
    else
    {
        NSLog(@"BLE_Pair.plist create default!" );
        pairedDeviceDict = [[NSMutableDictionary alloc] init];
    }
    
    bool bHadpairInfo;
    NSUUID *pairDevUUID;
    NSArray *peripheralarray;
    if ([pairedDeviceDict count] > 0)
    {
        //[self.connectButton setUserInteractionEnabled:NO];
        
        NSArray *keys = [pairedDeviceDict allKeys];
        for (id key in keys)
        {
            NSLog(@"Dic %@ - %@", key, [pairedDeviceDict objectForKey:key]);
        }

        NSString *pairUUIDstring = [pairedDeviceDict objectForKey:[NSString stringWithFormat:@"%s", BLE_WATCH_NAME]];
        if (pairUUIDstring != nil)
        {
            pairDevUUID = [[NSUUID UUID] initWithUUIDString:pairUUIDstring];
            NSLog(@"create the pairuuid string %@",pairDevUUID);

        }
        
        peripheralarray = [self.cm retrievePeripheralsWithIdentifiers:@[pairDevUUID]];
        
        if ([peripheralarray count] > 0)
            bHadpairInfo = true;
        else
            bHadpairInfo = false;
    }
    else
    {
        bHadpairInfo = false;
    }
    
    if (bHadpairInfo)
    {
        self.currentPeripheral = [[AlfordCTSCTPeripheral alloc] initWithPeripheral:[peripheralarray objectAtIndex:0]  delegate:self];
    }
    else
    {
        [self.connectButton setUserInteractionEnabled:YES];
        [self _StartLookBLEDevice];
    }
}

-(void)check
{
    if ([self.sendTextField.text length]>=3)
    {
        self.sendTextField.text =[self.sendTextField.text substringWithRange:NSMakeRange(0,2 )] ;
    }
}
-(void)check2
{
    if ([self.sendTextField2.text length]>=3)
    {
        self.sendTextField2.text =[self.sendTextField2.text substringWithRange:NSMakeRange(0,2 )] ;
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)connectButtonPressed:(id)sender
{
	[self.sendTextField resignFirstResponder];
	[self.sendTextField2 resignFirstResponder];

	switch (self.state) {
		case IDLE:
			self.state = SCANNING;
			[self _StartLookBLEDevice];
			break;

		case SCANNING:
			self.state = IDLE;
			NSLog(@"Stopped scan");
			[self.connectButton setTitle:@"Connect" forState:UIControlStateNormal];
			[self.cm stopScan];
			break;


		case CONNECTED:
			self.state = IDLE;
			NSLog(@"Disconnect peripheral %@", self.currentPeripheral.peripheral.name);
			[self.connectButton setTitle:@"Disconnect" forState:UIControlStateNormal];
			[self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];
			break;
	}
}

- (IBAction)sendTextFieldEditingDidBegin:(id)sender {
    [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:2] atScrollPosition:UITableViewScrollPositionBottom animated:YES];
    //    [self.tableView setContentOffset:CGPointMake(0, 220) animated:YES];
}

- (IBAction)sendTextFieldEditingChanged:(id)sender {
    //[self.sendTextField setFont:[UIFont systemFontOfSize:2]];
    sendtext1=self.sendTextField.text;
    sendtext2=self.sendTextField2.text;
    sendint1=[self.sendTextField.text intValue];
    sendint2=[self.sendTextField2.text intValue];
    if (sendint1>24)
    {
        [self.sendTextField setBackgroundColor:[UIColor redColor]];
    }
    else
    {
        [self.sendTextField setBackgroundColor:[UIColor whiteColor]];
    }
}
- (IBAction)sendTextFieldEditingChanged2:(id)sender {
    sendtext1=self.sendTextField.text;
    sendtext2=self.sendTextField2.text;
    sendint1=[self.sendTextField.text intValue];
    sendint2=[self.sendTextField2.text intValue];
    if (sendint2>59)
    {
        [self.sendTextField2 setBackgroundColor:[UIColor redColor]];
    }
    else
    {
        [self.sendTextField2 setBackgroundColor:[UIColor whiteColor]];
    }
}


- (BOOL) textFieldShouldReturn:(UITextField *)textField
{
    [self sendButtonPressed:textField];
    return YES;
}

- (IBAction)CurrentTimePressed:(id)sender {
    
	Byte cts_date[10];
    NSDate *date = [NSDate date];
	NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
	NSDateComponents *comps = [[NSDateComponents alloc] init];
	
    NSInteger unitFlags = NSYearCalendarUnit |
    NSMonthCalendarUnit |
    NSDayCalendarUnit |
    NSWeekdayCalendarUnit |
    NSHourCalendarUnit |
    NSMinuteCalendarUnit |
    NSSecondCalendarUnit;
    
	comps = [calendar components:unitFlags fromDate:date];
	
	cts_date[0] = [comps year]&0x00FF;
	cts_date[1] = ([comps year]&0xFF00) >> 8;
    cts_date[2] = [comps month];
	cts_date[3] = [comps day];
	cts_date[4] = [comps hour];
    cts_date[5] = [comps minute];
	cts_date[6] = [comps second];
    cts_date[7] = [comps weekday];
	cts_date[8] = 0x00;
    cts_date[9] = 0x01;
    
    NSData *adata = [[NSData alloc] initWithBytes:cts_date length:10];
	NSLog(@"currenttimePressed %@", adata);
    
    [self.currentPeripheral writeCurrentTime:adata];
    
}

- (IBAction)DelBondBt:(id)sender {
    self.currentPeripheral.peripheral = nil;
    NSLog(@"del bond master");
    [pairedDeviceDict removeAllObjects];
    [pairedDeviceDict writeToFile:pairInfopath atomically:YES];
    self.DelBond.enabled=FALSE;
}

- (IBAction)sendButtonPressed:(id)sender {
    
	Byte cts_date[10];
    NSDate *date = [NSDate date];
	NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
	NSDateComponents *comps = [[NSDateComponents alloc] init];
	
	 NSInteger unitFlags = NSYearCalendarUnit | 
						   NSMonthCalendarUnit |
						   NSDayCalendarUnit | 
						   NSWeekdayCalendarUnit | 
						   NSHourCalendarUnit |
						   NSMinuteCalendarUnit |
						   NSSecondCalendarUnit;
						   
	comps = [calendar components:unitFlags fromDate:date];
	
	cts_date[0] = [comps year]&0x00FF;
	cts_date[1] = ([comps year]&0xFF00) >> 8;
    cts_date[2] = [comps month];
	cts_date[3] = [comps day];
	cts_date[4] = [comps hour];
    cts_date[5] = [comps minute];
	cts_date[6] = [comps second];
    cts_date[7] = [comps weekday];
	cts_date[8] = 0x00;
    cts_date[9] = 0x01;
    
	//cts_date[0] = 0xdd;
	//cts_date[1] = 0x07;
    //cts_date[2] = 0x0A;
	//cts_date[3] = 0x05;
	//cts_date[4] = 0x0d;
    //cts_date[5] = 0x16;
	//cts_date[6] = 0x21;
    //cts_date[7] = 0x06;
	//cts_date[8] = 0x00;
    //cts_date[9] = 0x01;
    

	
    [self.sendTextField resignFirstResponder];
    [self.sendTextField2 resignFirstResponder];
    sendtext1=self.sendTextField.text;
    sendtext2=self.sendTextField2.text;
    sendint1=[self.sendTextField.text intValue];
    sendint2=[self.sendTextField2.text intValue];
    if (self.sendTextField.text.length == 0 && self.sendTextField2.text.length == 0)
    {
        return;
    }
    else if(sendint1<=24 && sendint2<=59)
    {
        cts_date[4] = sendint1;
        cts_date[5] = sendint2;
	    cts_date[6] = 0x01;
        NSData *adata = [[NSData alloc] initWithBytes:cts_date length:10];
        NSLog(@"Alarm Clock: %@", adata);
        
        [self.currentPeripheral writeAlarm:adata];
    }
    else
    {
        self.sendTextField.text =[self.sendTextField.text substringWithRange:NSMakeRange(0,0 )] ;
        self.sendTextField2.text =[self.sendTextField2.text substringWithRange:NSMakeRange(0,0)] ;
    }
}

- (void) ReconnectTimeout:(NSTimer *)sender
{
    NSLog(@"Re connect peripheral timout");
    self.state = IDLE;
    //[self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];
    
}

- (void) didReadHardwareRevisionString:(NSString *)string
{
    [self addTextToConsole:[NSString stringWithFormat:@"Hardware revision: %@", string] dataType:LOGGING];
}

- (void) didReceiveData:(NSString *)string
{
    [self addTextToConsole:string dataType:RX];
}

- (void) addTextToConsole:(NSString *) string dataType:(ConsoleDataType) dataType
{
    NSString *direction;
    switch (dataType)
    {
        case RX:
            direction = @"RX";
            break;
            
        case TX:
            direction = @"TX";
            break;
            
        case LOGGING:
            direction = @"Log";
    }
    
    NSDateFormatter *formatter;
    formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"HH:mm:ss.SSS"];
    
    //[self.consoleTextView setText:[self.consoleTextView.text stringByAppendingFormat:@"[%@] %@: %@\n",[formatter stringFromDate:[NSDate date]], direction, string]];
    
}

- (void) centralManagerDidUpdateState:(CBCentralManager *)central
{
    if (central.state == CBCentralManagerStatePoweredOn)
    {
        [self.connectButton setEnabled:YES];
    }
    
}

- (void) centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    NSLog(@"Did discover peripheral %@", peripheral.name);
    
    if ([peripheral.name rangeOfString:@"watch" options:NSCaseInsensitiveSearch].location == NSNotFound)
        return;
    
    [self.cm stopScan];
    
    self.currentPeripheral = [[AlfordCTSCTPeripheral alloc] initWithPeripheral:peripheral delegate:self];
    
    [self.cm connectPeripheral:peripheral options:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey: [NSNumber numberWithBool:YES]}];
}

- (void) centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"Did connect peripheral %@", peripheral.name);
    
    [self addTextToConsole:[NSString stringWithFormat:@"Did connect to %@", peripheral.name] dataType:LOGGING];
    
    self.state = CONNECTED;
    [self.connectButton setTitle:@"Disconnect" forState:UIControlStateNormal];
    [self.sendButton setUserInteractionEnabled:YES];
    [self.sendTextField setUserInteractionEnabled:YES];
    [self.sendTextField2 setUserInteractionEnabled:YES];
    
    if ([reconntimer isValid])
    {
        NSLog(@"connect by reconnect");
        [reconntimer invalidate];
        [self.currentPeripheral didConnect];
        return;
    }

    NSString *uuidstring = [NSString stringWithFormat:@"%@",[peripheral.identifier UUIDString]];
    if ([pairedDeviceDict objectForKey:peripheral.name])
    {
        [pairedDeviceDict removeAllObjects];
    }
    [pairedDeviceDict setObject:uuidstring forKey:peripheral.name];
    [pairedDeviceDict writeToFile:pairInfopath atomically:YES];
    
    NSMutableDictionary *savedDataDic = [NSMutableDictionary dictionaryWithContentsOfFile:pairInfopath];
    NSArray *keys = [savedDataDic allKeys];
    for (id key in keys)
    {
        NSLog(@"Dic %@ - %@", key, [pairedDeviceDict objectForKey:key]);
        self.DelBond.enabled =TRUE;
    }
    
    if ([self.currentPeripheral.peripheral isEqual:peripheral])
    {
        [self.currentPeripheral didConnect];
    }
}

- (void) centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Did disconnect peripheral %@", peripheral.name);
    
    [self addTextToConsole:[NSString stringWithFormat:@"Did disconnect from %@", peripheral.name] dataType:LOGGING];
    
    NSUUID *deviceUUID= [[NSUUID alloc] init];
    
    
    if (self.state == CONNECTED)
    {
        //[self.cm connectPeripheral:self.currentPeripheral.peripheral  options:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey: [NSNumber numberWithBool:YES]}];
        
        //reconntimer = [NSTimer scheduledTimerWithTimeInterval:60.0 target:self selector:@selector(ReconnectTimeout:) userInfo:nil repeats:YES];
        //self.state = IDLE;
        //return;
    }
    
    self.state = IDLE;
    [self.connectButton setTitle:@"Re-connect" forState:UIControlStateNormal];
    [self.sendButton setUserInteractionEnabled:NO];
    [self.sendTextField setUserInteractionEnabled:NO];
    [self.sendTextField2 setUserInteractionEnabled:NO];
    
    if ([self.currentPeripheral.peripheral isEqual:peripheral])
    {
        NSLog(@"Did re connect");
        
        //[self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];
        
        //[self.connectButton sendActionsForControlEvents:UIControlEventTouchUpInside];

        
        [self.currentPeripheral didDisconnect];
        //[self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];
    }
}
- (void) handleFoundPeripheral:(CBPeripheral*) peripheral{
    NSAssert(NO, @"must be overridden");
}

- (void)centralManager:(CBCentralManager *)central didRetrieveConnectedPeripherals:(NSArray *)peripherals
{
    NSLog(@"didRetrieveConnectedPeripherals");
    /*for (CBPeripheral *peripheral in peripherals) {
        [handleFoundPeripheral:peripherals];
    }
    if([delegate respondsToSelector:@selector(centralManager:didRetrievePeripherals:)]){
        [delegate centralManager:central didRetrievePeripherals:peripherals];
    }*/
}

- (void)centralManager:(CBCentralManager *)central didRetrievePeripherals:(NSArray *)peripherals
{
     NSLog(@"didRetrievePeripherals");
    
}

-(NSArray*)retrieveConnectedPeripheralsWithServices:(NSArray *)serviceUUIDs{
    NSArray *result = [self.cm retrieveConnectedPeripheralsWithServices:serviceUUIDs];
    NSLog(@"entery retrieveConnectedPeripheralsWithServices");
    [self centralManager:self.cm didRetrievePeripherals:result];
    return result;
}

-(NSArray *)retrievePeripheralsWithIdentifiers:(NSArray *)identifiers{
    NSArray *result = [self.cm retrievePeripheralsWithIdentifiers:identifiers];
    NSLog(@"entery retrievePeripheralsWithIdentifiers");
    [self centralManager:self.cm didRetrievePeripherals:result];
    return result;
}

- (void)_StartLookBLEDevice
{
    self.state = SCANNING;
    
    NSLog(@"Started scan ...");
    [self.connectButton setTitle:@"Scanning ..." forState:UIControlStateNormal];
    
    
    if (self.currentPeripheral.peripheral)
    {
        NSLog(@"have bond");
        [self.cm connectPeripheral:self.currentPeripheral.peripheral options:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey: [NSNumber numberWithBool:YES]}];
    }
    else
    {
        [self.cm scanForPeripheralsWithServices:@[AlfordCTSCTPeripheral.CTSServiceUUID] options:@{CBCentralManagerScanOptionAllowDuplicatesKey: [NSNumber numberWithBool:NO]}];
        //[self.cm scanForPeripheralsWithServices:Nil options:@{CBCentralManagerScanOptionAllowDuplicatesKey: [NSNumber numberWithBool:NO]}];
    }
}

@end
